@@ -2,10 +2,13 @@ import json
 import logging
 import re
 import string
+import uuid
 from dataclasses import dataclass, field
+from datetime import datetime
 from functools import lru_cache
 from json import JSONDecodeError
 from random import Random
+from types import FunctionType
 from typing import Dict, Union, List
 
 from .exceptions import HttpFileException, PropertyNotFoundException
@@ -20,6 +23,18 @@ class Property:
     value: Union[str, None] = None
 
 
+def random_string_generator(size=4, chars=string.ascii_lowercase + string.digits):
+    """
+    Args:
+        size: size of string
+        chars: list of chars allowed
+
+    Returns:
+        str: randomString with length and allowed chars
+    """
+    return ''.join(PropertyProvider.random.choice(chars) for _ in range(size))
+
+
 def get_random_str(length=None):
     """
     Args:
@@ -33,7 +48,7 @@ def get_random_str(length=None):
     if not length:
         length = random.randint(1, 10)
     initial = random.choice(string.ascii_letters)
-    return initial + ''.join(random.choices(string.ascii_letters + string.digits, k=length - 1))
+    return initial + random_string_generator(length - 1, string.ascii_letters + string.digits)
 
 
 def get_random_int(length=None):
@@ -55,6 +70,20 @@ def get_random_bool(*_args):
     return random.choice(['true', 'false'])
 
 
+def get_uuid(*_args):
+    return str(uuid.uuid4())
+
+
+def get_random_slug(length):
+    if not length or length <= 1:
+        length = 4
+    return '-'.join(random_string_generator(size=get_random_int(1)) for _i in range(length))
+
+
+def get_timestamp(*_args):
+    return int(datetime.timestamp(datetime.now()))
+
+
 class PropertyProvider:
     """
         1. properties defined in file itself ({{a=10}})
@@ -70,16 +99,23 @@ class PropertyProvider:
     """
     random = Random()
     var_regex = re.compile(r'{{(?P<var>.*?)}}', re.DOTALL)
-    random_string_regex = re.compile(
-        r'.*?(?P<category>\$randomStr|\$randomInt|\$randomBool|\$randomFloat)(?P<length>:\d*)?')
-
-    rand_map = {
+    rand_map: Dict[str, FunctionType] = {
         '$randomStr': get_random_str,
         '$randomInt': get_random_int,
         '$randomFloat': get_random_float,
-        '$randomBool': get_random_bool
+        '$randomBool': get_random_bool,
+        '$guid': get_uuid,
+        '$uuid': get_uuid,
+        '$timestamp': get_timestamp,
+        '$randomLoremSlug': get_random_slug,
+        '$randomSlug': get_random_slug,
     }
 
+    # random_string_regex = re.compile(
+    #     r'.*?(?P<category>\$randomStr|\$randomInt|\$randomBool|\$randomFloat|\$guid)(?P<length>:\d*)?')
+    random_string_regex = re.compile(
+        r'.*?(?P<category>' + "|".join("\\" + key for key in rand_map) + ')(?P<length>:\d*)?')
+
     def __init__(self, property_file=""):
         self.command_properties = {}
         self.env_properties = {}
@@ -128,6 +164,12 @@ class PropertyProvider:
             set(key for key in self.infile_properties if
                 self.infile_properties[key].value is not None or PropertyProvider.is_random_key(key)))
 
+    def get_all_properties_variables(self):
+        # TODO
+        d = dict()
+        d.update(self.command_properties)
+        return d
+
     @staticmethod
     def is_random_key(key):
         return any(key.startswith(rand_category_name) for rand_category_name in PropertyProvider.rand_map)
